<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <!--
  		@desc cPanel 9/10
       -->

  <!-- new client & reseller's contact name prefixes -->
  <xsl:param name='client-name-prefix'>client </xsl:param>
  <xsl:param name='reseller-name-prefix'>reseller </xsl:param>

  <!-- maximum significant value for the disk space / traffic limit.
       values exceeding that would be counted as 'unlimited' -->
  <xsl:variable name='max_limit_value'
    select='format-number(999999*1024*1024, "#")'/>

  <!--
       Conversion from cPanel 9 business model to Plesk 7 business model.
       -->

  <xsl:output indent="yes" encoding="UTF-8"/>

  <xsl:template name="cPanel9dump" match="cPanel9dump">

    <xsl:text disable-output-escaping="yes">&lt;!DOCTYPE plesk7dump SYSTEM 'Plesk70.dtd'&gt;</xsl:text>

    <plesk7dump>
      <xsl:call-template name="ipmap"/>

      <!-- convert cPanel resellers into the Plesk clients with domain-creation capability -->
      <xsl:apply-templates select="reseller[@name!='']"/>

      <!-- convert cPanel accounts, belonging to default reseller to Plesk clients without domain creation capability -->
      <xsl:apply-templates select="reseller[@name='']/account"/>
    </plesk7dump>

  </xsl:template>

  <!--
       Assemble the IP map for migration from the cPanel to Plesk.
       -->
  <xsl:key name="ip-shared"    match="reseller/account[(domain/anonftp/@pub!='true' or (string-length(domain)>0 and not(domain/anonftp/@pub))) and string-length(ip)>0]" use="ip"/>
    <xsl:key name="ip-exclusive" match="reseller/account[domain/anonftp/@pub ='true' and string-length(ip)>0]" use="ip"/>

      <xsl:template name="ipmap">
        <map>
          <shared>
            <!-- map usual domains on shared IP address -->
            <xsl:for-each select="reseller/account[generate-id(.) = generate-id(key('ip-shared', ip))]">
              <ip id="{generate-id(ip)}">
                <old><xsl:value-of select="ip"/></old>
                <new><xsl:value-of select="ip"/></new>
              </ip>
            </xsl:for-each>
          </shared>
          <exclusive>
            <!-- map anonymousftp capable domains on exclusive IP addresses -->
            <xsl:for-each select="reseller/account[generate-id(.) = generate-id(key('ip-exclusive', ip))]">
              <ip id="{generate-id(ip)}">
                <old><xsl:value-of select="ip"/></old>
                <new><xsl:value-of select="ip"/></new>
              </ip>
            </xsl:for-each>
          </exclusive>
        </map>
      </xsl:template>

      <!--
           Convert cPanel resellers into Plesk's clients with domain-creation capabilities.
           Resold resellers accounts are converted to domain users.
           This template not handle default reseller.
           -->
      <xsl:template name="reseller" match="reseller[string-length(@name)>0]">

        <xsl:variable name="name" select="@name"/>
        <xsl:if test='string-length($name)=0'>
          <xsl:message terminate="yes">Can&apos;t determine the reseller&apos;s name</xsl:message>
        </xsl:if>

        <client name="{$name}" contact="{concat($reseller-name-prefix, $name)}">
          <xsl:copy-of select='account[@name=$name]/@email[.!=""]'/>

          <!--
               Login password - use the element ID.
               Actually the user can't know it - so it needs to be emailed to user
               after restore.
               -->
          <xsl:call-template name='password'>
            <xsl:with-param name='password'      select='generate-id()'/>
            <xsl:with-param name='password_type' select='"plain"'/>
          </xsl:call-template>

          <!-- Handle limits/permissions -->
          <xsl:call-template name="reseller-limits"/>
          <xsl:call-template name="account-permissions"/>
          <xsl:call-template name="reseller-permissions"/>

          <!--
               Assemble IP pool for the reseller.
               Determine the IP addresses ID used for given account.
               Get the IP address value and search the IP object used for
               generating the ID for this IP address.
               We should search exact type of IPs - shared or exclusive.
               We use the same algoritm as in ip map assembling.

               Removing duplicate IP entries.
               -->

          <ip_pool>

            <!-- shared IP addresses -->
            <xsl:for-each select='account[domain/anonftp/@pub!="true" or (string-length(domain)>0 and not(domain/anonftp/@pub))]'>
              <xsl:variable name="ip" select="ip"/>
              <client_ip ip_ref="{generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-shared', ip)) and ip=$ip]/ip)}"/>
            </xsl:for-each>

            <!-- exclusive IP addresses -->
            <xsl:for-each select='account[domain/anonftp/@pub="true"]'>
              <xsl:variable name="ip" select="ip"/>
              <client_ip ip_ref="{generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-exclusive', ip)) and ip=$ip]/ip)}"/>
            </xsl:for-each>

            <!-- empty IP addresses (how do they occure? disabled domains or something...) -->
            <xsl:for-each select="account[string-length(ip)=0]">
              <client_ip ip_ref="{generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-shared', ip))]/ip)}"/>
            </xsl:for-each>

          </ip_pool>

          <!--
               Create domains.
               Some domains may not have IP address (disabled or something)
               If such domain is found - assign him an arbitrary shared IP address.
               TODO: investigate why there are such domains and what we can do with it.
               -->
          <xsl:for-each select="account">

            <xsl:variable name="ip" select="ip"/>

            <!-- register the domain for current reseller -->
            <xsl:choose>

              <xsl:when test='string-length(ip)=0'>
                <xsl:variable name='ipref'     select='generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key("ip-shared", ip))]/ip)'/>
                <xsl:variable name="quota"     select="limit[@name='disk_space']"/>
                <xsl:variable name="du_passwd" select="generate-id(../account[.=current() and @name!=$name])"/>
                <xsl:variable name="du_email"   select="../account[.=current() and @name!=$name]/@email"/>
                <xsl:variable name="du_contact" select="../account[.=current() and @name!=$name]/@name"/>

                <xsl:for-each select='domain'>
                  <xsl:call-template name="domain">
                    <xsl:with-param name="ipref"     select="$ipref"/>
                    <xsl:with-param name="quota"     select="$quota"/>
                    <xsl:with-param name="du_passwd" select="$du_passwd"/>
                    <xsl:with-param name="du_email"   select="$du_email"/>
                    <xsl:with-param name="du_contact" select="$du_contact"/>
                  </xsl:call-template>
                </xsl:for-each>
              </xsl:when>

              <xsl:otherwise>

                <xsl:choose>
                  <!-- watch for exclusive IP -->
                  <xsl:when test="domain/anonftp[@pub='true']">
                    <xsl:variable name="ipref"     select="generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-exclusive', ip)) and ip=$ip]/ip)"/>
                    <xsl:variable name="quota"     select="limit[@name='disk_space']"/>
                    <xsl:variable name="du_passwd" select="generate-id(../account[.=current() and @name!=$name])"/>
                    <xsl:variable name="du_email"   select="../account[.=current() and @name!=$name]/@email"/>
                    <xsl:variable name="du_contact" select="../account[.=current() and @name!=$name]/@name"/>

                    <xsl:for-each select='domain'>
                      <xsl:call-template name="domain">
                        <xsl:with-param name="ipref" select="$ipref"/>
                        <xsl:with-param name="quota" select="$quota"/>
                        <xsl:with-param name="du_passwd" select="$du_passwd"/>
                        <xsl:with-param name="du_email"   select="$du_email"/>
                        <xsl:with-param name="du_contact" select="$du_contact"/>
                      </xsl:call-template>
                    </xsl:for-each>

                  </xsl:when>
                  <!-- Concern the shared IP address -->
                  <xsl:otherwise>
                    <xsl:variable name="ipref" select="generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-shared', ip)) and ip=$ip]/ip)"/>
                    <xsl:variable name="quota" select="limit[@name='disk_space']"/>
                    <xsl:variable name="du_passwd" select="generate-id(../account[.=current() and @name!=$name])"/>
                    <xsl:variable name="du_email"   select="../account[.=current() and @name!=$name]/@email"/>
                    <xsl:variable name="du_contact" select="../account[.=current() and @name!=$name]/@name"/>

                    <xsl:for-each select='domain'>
                      <xsl:call-template name="domain">
                        <xsl:with-param name="ipref" select="$ipref"/>
                        <xsl:with-param name="quota" select="$quota"/>
                        <xsl:with-param name="du_passwd" select="$du_passwd"/>
                        <xsl:with-param name="du_email"   select="$du_email"/>
                        <xsl:with-param name="du_contact" select="$du_contact"/>
                      </xsl:call-template>
                    </xsl:for-each>

                  </xsl:otherwise>
                </xsl:choose>

              </xsl:otherwise>
            </xsl:choose>

          </xsl:for-each>

        </client>

      </xsl:template>

      <!--
           Converts cPanel account into Plesk's client
           -->
      <xsl:template name="client" match="account">

        <xsl:element name="client">

          <xsl:copy-of select='@name[.!=""]'/>
          <xsl:attribute name='contact'>
            <xsl:value-of select='concat($client-name-prefix, @name)'/>
          </xsl:attribute>
          <xsl:copy-of select='@email[.!=""]'/>

          <!--
               Login password - use the element ID.
               Actually the user can't know it - so it needs to be emailed to user
               after restore.
               -->
          <xsl:call-template name='password'>
            <xsl:with-param name='password' select='generate-id()'/>
            <xsl:with-param name='password_type' select='"plain"'/>
          </xsl:call-template>

          <!-- Handle limits -->
          <xsl:call-template name="account-limits"/>

          <!-- Generate permissions -->
          <xsl:call-template name="account-permissions"/>

          <!--
               Determine the IP address ID used for given account.
               Get the IP address value and search the IP object used for
               generating the ID for this IP address.
               We should search exact type of IPs - shared or exclusive.
               We use the same algoritm as in ipmap assembling.

               To not repeat this computations during domain transformation
               call domain template with IP address ID.
               -->

          <xsl:variable name="ip" select="ip"/>

          <xsl:choose>

            <xsl:when test='string-length(ip)=0'>
              <xsl:variable name='ipref' select='generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key("ip-shared", ip))]/ip)'/>
              <ip_pool>
                <client_ip ip_ref="{$ipref}"/>
              </ip_pool>

              <!-- To not repeat this computations during domain transformation call domain template right now -->
              <xsl:variable name="quota" select="limit[@name='disk_space']"/>

              <xsl:for-each select="domain">
                <xsl:call-template name="domain">
                  <xsl:with-param name="ipref" select="$ipref"/>
                  <xsl:with-param name="quota" select="$quota"/>
                </xsl:call-template>
              </xsl:for-each>

            </xsl:when>

            <xsl:otherwise>

              <xsl:choose>

                <!-- Concern the exclusive IP address -->
                <xsl:when test="domain/anonftp[@pub='true']">

                  <xsl:variable name="ipref" select="generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-exclusive', ip)) and ip=$ip]/ip)"/>
                  <ip_pool>
                    <client_ip ip_ref="{$ipref}"/>
                  </ip_pool>

                  <!-- To not repeat this computations during domain transformation call domain template right now -->
                  <xsl:variable name="quota" select="limit[@name='disk_space']"/>

                  <xsl:for-each select="domain">
                    <xsl:call-template name="domain">
                      <xsl:with-param name="ipref" select="$ipref"/>
                      <xsl:with-param name="quota" select="$quota"/>
                    </xsl:call-template>
                  </xsl:for-each>

                </xsl:when>

                <!-- Concern the shared IP address -->
                <xsl:otherwise>

                  <xsl:variable name="ipref" select="generate-id(/cPanel9dump/reseller/account[generate-id(.) = generate-id(key('ip-shared', ip)) and ip=$ip]/ip)"/>
                  <ip_pool>
                    <client_ip ip_ref="{$ipref}"/>
                  </ip_pool>

                  <!-- To not repeat this computations during domain transformation call domain template right now -->
                  <xsl:variable name="quota" select="limit[@name='disk_space']"/>

                  <xsl:for-each select="domain">
                    <xsl:call-template name="domain">
                      <xsl:with-param name="ipref" select="$ipref"/>
                      <xsl:with-param name="quota" select="$quota"/>
                    </xsl:call-template>
                  </xsl:for-each>

                </xsl:otherwise>
              </xsl:choose>

            </xsl:otherwise>
          </xsl:choose>

        </xsl:element>

      </xsl:template>

      <!-- Handle account limits -->
      <xsl:template name="account-limits">

        <!-- Some cPanel limits may be directly mapped into Plesk's -->
        <xsl:copy-of select="limit[@name='max_traffic']"  />
        <xsl:copy-of select="limit[@name='max_maillists']"/>
        <xsl:copy-of select="limit[@name='max_box']"      />
        <xsl:copy-of select="limit[@name='max_db']"       />
        <xsl:copy-of select="limit[@name='max_subdom']"   />
        <xsl:copy-of select="limit[@name='disk_space']"   />

        <!-- Limit on ftp users is transformed to limit on webusers -->
        <xsl:if test="limit[@name='max_ftpusers']">
          <xsl:call-template name="limit">
            <xsl:with-param name="limit" select='"max_wu"'/>
            <xsl:with-param name="value" select="limit[@name='max_ftpusers']"/>
          </xsl:call-template>
        </xsl:if>
        <!-- Limits on addon and parked domains don't mean anything in Plesk
             And therefore silently skipped. -->
      </xsl:template>

      <!--
           Outputs domain limits setted to zero (used for addon domains).
           Disk space is set to 50 Kb (for convenience)
           -->
      <xsl:template name='domain-limits-zero'>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_traffic</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_maillists</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_box</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_db</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_subdom</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>disk_space</xsl:with-param>
          <xsl:with-param name='value' select='1024*50'/>
        </xsl:call-template>
        <xsl:call-template name='limit'>
          <xsl:with-param name='limit'>max_wu</xsl:with-param>
          <xsl:with-param name='value' select='0'/>
        </xsl:call-template>
      </xsl:template>

      <!--
           "Clever" domain limits processing for those limits, actual usage of what
           can't be calculated (traffic/disk space) - tries to satisfy reseller's
           limits by dividing available reseller's resources among domains, which
           have not such limit specified.

           Should be applied inside <domain>
           Default reseller has no any limits specified so this scheme works fine
           for default reseller's accounts.

           Parameters:
           name - limit name
           -->
      <xsl:template name='domain-limit-shared'>

        <xsl:param name='name'/>

        <!--
             During the domain limits processing there are following situations may
             occure:

             1 reseller has limited traffic or disk space.
             1.1 domain-owning account has its own limit (copy account's limit
             value into domain)
             1.2 domain-owning account has no limit specified or limit is unlimited
             (in this case we should divide free reseller's limit among such
             domains)

             2 reseller has no limits for traffic or disk space
             2.1 domain-owning account has its own limit (copy the limit's value)
             2.2 domain-owning account has no limit specified or limit is unlimited
             (don't doing anything - reseller is unlimited in resources anyway)

             Algorithm:
             1. if resource is limited in account - copy it's value
             2. if resource is unlimited in account - check for reseller's
             limitations and divide available reseller's resources among such
             accounts
             -->

        <xsl:variable name='limit' select='format-number(../limit[@name=$name], "#")'/>

        <xsl:choose>

          <!-- account has it's own significant limits. NOTE: as Plesk anyway
               doesn't allow limit more then 10^9 and as cPanel doesn't have
               official unlimited value (very large numbers are used for it)
               we suppose limits more than $max_limit_value as unlimited -->
          <xsl:when test='($limit != "") and ($limit != -1) and ($limit &lt; $max_limit_value)'>

            <!-- leave some space for addon and parked domains (50K for each) -->
            <xsl:variable name='addon'  select='count(addondomain)'/>
            <xsl:variable name='parked' select='count(subdomain/addondomain)'/>
            <xsl:variable name='space'  select='$limit - 50*1024*($addon + $parked)'/>

            <xsl:choose>
              <!-- if there are enough space for reservation for addon/parked domains -->
              <xsl:when test='$space > 0'>
                <xsl:call-template name='limit'>
                  <xsl:with-param name='limit' select='$name'/>
                  <xsl:with-param name='value' select='format-number($space, "#")'/>
                </xsl:call-template>
              </xsl:when>
              <!-- if space isn't enough -->
              <xsl:otherwise>
                <xsl:call-template name='limit'>
                  <xsl:with-param name='limit' select='$name'/>
                  <xsl:with-param name='value' select='format-number($space, "#")'/>
                </xsl:call-template>
              </xsl:otherwise>
            </xsl:choose>

          </xsl:when>

          <!-- either account has no limits specified, or limit is specified
               as 'unlimited' -->
          <xsl:otherwise>

            <!-- watch for reseller's limit -->
            <xsl:variable name='r_limit' select='format-number(../../limit[@name=$name], "#")'/>

            <xsl:if test='($r_limit != "") and ($r_limit != -1) and ($r_limit &lt; $max_limit_value)'>

              <!-- but reseller has the same parameter limited - so we
                   should count free reseller's resources and divide them among
                   the domains, which has no limits specified -->

              <!-- count summary reserved limits for this reseller -->
              <xsl:variable name='reserved' select='sum(../../account/limit[(@name=$name) and (. != -1) and (. != "") and (. &lt; $max_limit_value)])'/>
              <xsl:variable name='free'     select='$r_limit - $reserved'/>

              <xsl:choose>
                <!-- check that free resources are available -->
                <xsl:when test='$free &lt;= 0'>

                  <!-- reseller doesn't have enough limits to recreate
                       neccessary objects -->
                  <xsl:message>
                    <xsl:text>How do you use cPanel if it allows to exceed the reseller's "</xsl:text>
                    <xsl:value-of select='../../@name'/>
                    <xsl:text>" limit (</xsl:text>
                    <xsl:value-of select='$name'/>
                    <xsl:text> = </xsl:text>
                    <xsl:value-of select='$r_limit'/>
                    <xsl:text>) in it's resold accounts (</xsl:text>
                    <xsl:value-of select='$reserved'/>
                    <xsl:text>)? Plesk wouldn't allow it... You'll have problems with deployment of the domain </xsl:text>
                    <xsl:value-of select='@name'/>
                  </xsl:message>

                </xsl:when>

                <xsl:otherwise>
                  <!-- count a quontity of the reseller's accounts
                       (domains), having no limits specified (among which
                       reseller's limits should be divided) -->
                  <xsl:variable name='unhandled_domains'
                    select='count(../../account[(limit[@name=$name] = "") or (limit[@name=$name] = -1) or (limit[@name=$name] &gt; $max_limit_value)])'/>

                  <!-- generate limit for domain slightly less, then it
                       may	be - to have resources to create other domains
                       (standard forwarding domains) for this reseller -->

                  <!-- WARNING: trick with format-number() is needed due
                       to the bug in libxslt which causes trash output of
                       floor/round/ceiling for big numbers (10^10). When this
                       bug	eleminated it's safe to replace format-number() by
                       floor()	(Thanks to Ilnar for the sollution :) -->

                  <xsl:call-template name='limit'>
                    <xsl:with-param name='limit' select='$name'/>
                    <xsl:with-param name='value' select='format-number(($free div $unhandled_domains) - 1048576, "#")'/>
                  </xsl:call-template>

                </xsl:otherwise>

              </xsl:choose>

            </xsl:if>

          </xsl:otherwise>
        </xsl:choose>

      </xsl:template>

      <!--
           Handle reseller limits. There are 3 limits currently: number of accounts,
           traffic and disk space. Concerning traffic and disk space: there is an
           unclear situation when reseller's account has an unlimited limit, but
           reseller is limited in a number of traffic, which it can resold. What
           should I do with it? It seems preferable to limit such reseller by it's
           available reseller's limits (i.e. limit it by a number of resource it
           can resold).
           -->
      <xsl:template name="reseller-limits">
        <xsl:variable name='reseller-name' select='@name'/>

        <!-- Maximum accounts limit (if not -1) is transformed into maximum
             domains. We also should add quantity of addon/parked domains to not
             break anything -->
        <xsl:if test="(limit[@name='max_accounts']) and (limit[@name='max_accounts'] != -1)">
          <!-- adds 1 for account of the reseller itself -->
          <xsl:variable name='account-limit' select='limit[@name="max_accounts"] + 1'/>

          <!-- add quantity of addon/parked domains to the creation account limit -->
          <xsl:variable name="addon-domains"  select="count(account/domain/addondomain)"/>
          <xsl:variable name="parked-domains" select="count(account/domain/subdomain/addondomain)"/>

          <xsl:call-template name="limit">
            <xsl:with-param name="limit">max_dom</xsl:with-param>
            <xsl:with-param name="value" select="$account-limit + $parked-domains + $addon-domains"/>
          </xsl:call-template>

        </xsl:if>

        <!-- at this point we may correct the reseller's disk space and traffic
             limits -->
        <xsl:variable name='traffic_limit' select='format-number(limit[@name="max_traffic"], "#")'/>

        <xsl:if test='($traffic_limit != "") and ($traffic_limit != -1) and ($traffic_limit &lt; $max_limit_value)'>

          <!-- count summary reserved limits -->
          <xsl:variable name='reserved_traffic' select='sum(account/limit[(@name="max_traffic") and (. != -1) and (. != "") and (. &lt; $max_limit_value)])'/>

          <!-- if there is not enough resources to deploy accounts, which
               have their space reserved we should increase reseller's limit to at
               least the required value + 1Gb just in case... -->
          <xsl:choose>
            <xsl:when test='$reserved_traffic &gt;= $traffic_limit'>

              <xsl:call-template name='limit'>
                <xsl:with-param name='limit' select='"max_traffic"'/>
                <xsl:with-param name='value' select='format-number($reserved_traffic + 1024*1024*1024, "#")'/>
              </xsl:call-template>

            </xsl:when>
            <xsl:otherwise>

              <xsl:call-template name='limit'>
                <xsl:with-param name='limit' select='"max_traffic"'/>
                <xsl:with-param name='value' select='$traffic_limit'/>
              </xsl:call-template>

            </xsl:otherwise>
          </xsl:choose>

        </xsl:if>

        <xsl:variable name='space_limit' select='format-number(limit[@name="disk_space"], "#")'/>

        <xsl:if test='($space_limit != "") and ($space_limit != -1) and ($space_limit &lt; $max_limit_value)'>

          <!-- count summary reserved limits -->
          <xsl:variable name='reserved_space' select='sum(account/limit[(@name="disk_space") and (. != -1) and (. != "") and (. &lt; $max_limit_value)])'/>

          <!-- if there is not enough resources to deploy accounts, which
               have their space reserved we should increase reseller's limit to at
               least the required value + 10Mb just in case... -->
          <xsl:choose>
            <xsl:when test='$reserved_space &gt;= $space_limit'>

              <xsl:call-template name='limit'>
                <xsl:with-param name='limit' select='"disk_space"'/>
                <xsl:with-param name='value' select='format-number($reserved_space + 1024*1024*10, "#")'/>
              </xsl:call-template>

            </xsl:when>
            <xsl:otherwise>

              <xsl:call-template name='limit'>
                <xsl:with-param name='limit' select='"disk_space"'/>
                <xsl:with-param name='value' select='$space_limit'/>
              </xsl:call-template>

            </xsl:otherwise>
          </xsl:choose>

        </xsl:if>

      </xsl:template>

      <!-- Handle permissions -->
      <xsl:template name="account-permissions">

        <!-- Some neccessary permissions, which exist on cPanel -->
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_subdomains</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_dns</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_log</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_crontab</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_anonftp</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="permission">
          <xsl:with-param name="permission">manage_maillists</xsl:with-param>
        </xsl:call-template>

      </xsl:template>

      <!-- Handle reseller permissions -->
      <xsl:template name="reseller-permissions">
        <xsl:variable name='reseller-name' select='@name'/>

        <xsl:if test="permission/@name='create_accounts' or permission/@name='create_domains'">
          <xsl:call-template name="permission">
            <xsl:with-param name="permission">create_domains</xsl:with-param>
          </xsl:call-template>
        </xsl:if>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

           Convert cPanel domain into the Plesk one

           Parameters:
           ipref - ip address used for this domain reference
           quota - account's disk quota (if any)
           du_passwd - domain user password (if any)
           - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="domain">

        <xsl:param name="ipref"/>
        <xsl:param name="quota"/>
        <xsl:param name="du_passwd"/>
        <xsl:param name="du_email"/>
        <xsl:param name="du_contact"/>

        <!-- - - - - - - - - - - - - - - - - - - - - - - - - -
             Restore the "plain" business objects structure.
             All exceptions are handled later.
             - - - - - - - -  - - - - - - - - - - - - - - - - - - -->
        <xsl:element name="domain">
          <xsl:attribute name="name"><xsl:value-of select="@name"/></xsl:attribute>

          <!-- used ip -->
          <domain_ip ip_ref="{$ipref}"/>

          <!-- disk space and traffic limits require special processing -->
          <xsl:call-template name='domain-limit-shared'>
            <xsl:with-param name='name' select='"max_traffic"'/>
          </xsl:call-template>
          <xsl:call-template name='domain-limit-shared'>
            <xsl:with-param name='name' select='"disk_space"'/>
          </xsl:call-template>

          <!-- Continue mess with the limits:
               other significant are subdomains/webusers/mailboxes limits -->
          <xsl:copy-of select='../limit[@name="max_subdom" and . != ""]'/>
          <xsl:copy-of select='../limit[@name="max_box" and . != ""]'/>

          <xsl:call-template name='limit'>
            <xsl:with-param name='limit' select='"max_wu"'/>
            <xsl:with-param name='value' select='../limit[@name="max_ftpusers"]'/>
          </xsl:call-template>

          <!-- mail (mail subsystem of the domain) -->
          <xsl:element name="mailsystem">
            <xsl:apply-templates select="mail"/>
          </xsl:element>

          <!-- dnsrec -->
          <!-- database -->
          <xsl:copy-of select="database"/>

          <!-- maillist -->
          <!-- traffic -->
          <!-- status -->
          <!-- certificate -->
          <!-- domainuser -->
          <xsl:if test="string-length($du_passwd)>0">
            <domainuser>
              <xsl:if test="string-length($du_contact)>0">
                <xsl:attribute name="contact">
                  <xsl:value-of select="$du_contact"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:if test="string-length($du_email)>0">
                <xsl:attribute name="email">
                  <xsl:value-of select="$du_email"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:call-template name="password">
                <xsl:with-param name="password" select="$du_passwd"/>
                <xsl:with-param name="password_type" select="'plain'"/>
              </xsl:call-template>
            </domainuser>
          </xsl:if>

          <!-- shosting/fhosting/phosting -->
          <xsl:call-template name="hosting">
            <xsl:with-param name="quota" select="$quota"/>
          </xsl:call-template>
        </xsl:element>

        <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
             There is a very special issue about addon domains:
             they should be transformed into separate domains with
             standard forwarding pointing to Plesk's domains...
             This is performed for addon and parked domains.
             - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->

        <!-- addon domains -->
        <xsl:for-each select="subdomain/addondomain">
          <xsl:call-template name="addondomain">
            <xsl:with-param name="domain" select="concat(../@name, '.', ../../@name)"/>
            <xsl:with-param name="ipref" select="$ipref"/>
            <!-- if domain user is specified for domain - duplicate it -->
            <xsl:with-param name='du_passwd'  select='$du_passwd'/>
            <xsl:with-param name='du_email'   select='$du_email'/>
            <xsl:with-param name='du_contact' select='$du_contact'/>
          </xsl:call-template>
        </xsl:for-each>

        <!-- parked domains -->
        <xsl:for-each select="addondomain">
          <xsl:call-template name="addondomain">
            <xsl:with-param name="domain" select="../@name"/>
            <xsl:with-param name="ipref" select="$ipref"/>
            <!-- if domain user is specified for domain - duplicate it -->
            <xsl:with-param name='du_passwd'  select='$du_passwd'/>
            <xsl:with-param name='du_email'   select='$du_email'/>
            <xsl:with-param name='du_contact' select='$du_contact'/>
          </xsl:call-template>
        </xsl:for-each>

      </xsl:template>

      <!--
           cPanel domains always have hosting. Transform it into
           the Plesk's model.
           Generate the hosting element with all features enabled._
           Parameters:
           quota - if account has disk space limit
           -->
      <xsl:template name="hosting">

        <xsl:param name="quota"/>

        <xsl:element name="phosting">

          <xsl:copy-of select="@cid_docroot[.!='']"/>
          <xsl:copy-of select="@cid_cgi[.!='']"/>
          <xsl:attribute name="https"  >true</xsl:attribute>
          <xsl:attribute name="fp"     >true</xsl:attribute>
          <xsl:attribute name="fpssl"  >true</xsl:attribute>
          <xsl:attribute name="fpauth" >true</xsl:attribute>
          <xsl:attribute name="webstat">true</xsl:attribute>
          <xsl:attribute name="errdocs">true</xsl:attribute>

          <!-- sysuser -->
          <xsl:choose>
            <xsl:when test="$quota">
              <xsl:call-template name="sysuser">
                <xsl:with-param name="name"     select="ftpuser[not(@directory)]/@name"/>
                <xsl:with-param name="password" select="ftpuser[not(@directory)]/@password"/>
                <xsl:with-param name="quota"    select="$quota"/>
              </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="sysuser">
                <xsl:with-param name="name"     select="ftpuser[not(@directory)]/@name"/>
                <xsl:with-param name="password" select="ftpuser[not(@directory)]/@password"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>

          <!-- scripting -->
          <xsl:call-template name="scripting-all"/>

          <!-- fpuser -->
          <fpuser name="{ftpuser[not(@directory)]/@name}">
            <password type='encrypted'><xsl:value-of select="ftpuser[not(@directory)]/@password"/></password>
          </fpuser>

          <!-- logrotation -->
          <!-- anonftp - needs no structural transformation -->
          <xsl:copy-of select="anonftp"/>

          <!-- protected directories (pdir) - directory users should be modified -->
          <xsl:for-each select="pdir">

            <xsl:element name="pdir">
              <xsl:copy-of select="@name[.!='']"/>
              <xsl:copy-of select="@title[.!='']"/>

              <xsl:for-each select="pduser">
                <pduser name="{@name}">
                  <xsl:if test="@password">
                    <password type="encrypted">
						<xsl:if test="@encoding">
							<xsl:attribute name="encoding">
								<xsl:value-of select="@encoding"/>
							</xsl:attribute>
						</xsl:if>
                      <xsl:value-of select="@password"/>
                    </password>
                  </xsl:if>
                </pduser>
              </xsl:for-each>

            </xsl:element>

          </xsl:for-each>

          <!-- web users (webuser) -->
          <xsl:apply-templates select="ftpuser[@directory]"/>

          <!-- subdomains (subdomain) -->
          <xsl:apply-templates select="subdomain"/>

        </xsl:element>

      </xsl:template>

      <!--
           Outputs subdomain element.
           -->
      <xsl:template name="subdomain" match="subdomain">
        <xsl:element name="subdomain">
          <xsl:copy-of select="@name[.!='']"/>
          <xsl:copy-of select="@cid_docroot[.!='']"/>
          <xsl:copy-of select="@cid_cgi[.!='']"/>

          <!-- may contain separate ftp user -->
          <xsl:if test="ftpuser">
            <xsl:call-template name="sysuser">
              <xsl:with-param name="name" select="ftpuser/@name"/>
              <xsl:with-param name="password" select="ftpuser/@password"/>
            </xsl:call-template>
          </xsl:if>

          <!-- enable scripting options -->
          <xsl:call-template name="scripting-all"/>

          <!-- there may be mail subsystem - unfortunately it should be ignored -->
        </xsl:element>
      </xsl:template>

      <!--
           Convert addondomain into the Plesk's domain with the standard
           forwarding hosting.

           Parameters:
           domain - FQDN of the subdomain for the addon domain and
           FQDN of the domain for the parked domain. I.e. FQDN of
           the domain for which given addon domain is belongs to.
           ipref  - id of the ip address used for domain creation
           du_passwd  - domain user's password (if any)
           du_email   - domain user's email (if any)
           du_contact - domain user's contact name (if any)
           -->
      <xsl:template name="addondomain">

        <xsl:param name="domain"/>
        <xsl:param name="ipref"/>
        <xsl:param name="du_passwd"/>
        <xsl:param name="du_email"/>
        <xsl:param name="du_contact"/>

        <xsl:variable name="addondomain" select='.'/>

        <domain name="{$addondomain}">
          <domain_ip ip_ref="{$ipref}"/>
          <xsl:call-template name='domain-limits-zero'/>

          <!-- domainuser (if any) -->
          <xsl:if test="string-length($du_passwd)>0">
            <domainuser>
              <xsl:if test="string-length($du_contact)>0">
                <xsl:attribute name="contact">
                  <xsl:value-of select="$du_contact"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:if test="string-length($du_email)>0">
                <xsl:attribute name="email">
                  <xsl:value-of select="$du_email"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:call-template name="password">
                <xsl:with-param name="password" select="$du_passwd"/>
                <xsl:with-param name="password_type" select="'plain'"/>
              </xsl:call-template>
            </domainuser>
          </xsl:if>

          <shosting>
            <xsl:value-of select="$domain"/>
          </shosting>
        </domain>

      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
           Converts cPanel's ftp user into Plesk's web user. Such transformation
           may be performed only for ftp users of main domain, which have
           their own directories. Subdomains and directories within subdomains
           can't be transformed in such a way.
           Directory name doesn't matter in this case, so silently ignored...
           If webuser has no password - generate a random one...
           - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="webuser" match="ftpuser">
        <xsl:element name="webuser">
          <xsl:copy-of select="@cid[.!='']"/>
          <xsl:choose>
            <xsl:when test='string-length(@password)=0'>
              <xsl:call-template name="sysuser">
                <xsl:with-param name="name"     select="@name"         />
                <xsl:with-param name="password" select="generate-id(.)"/>
              </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
              <xsl:call-template name="sysuser">
                <xsl:with-param name="name"     select="@name"    />
                <xsl:with-param name="password" select="@password"/>
              </xsl:call-template>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:call-template name="scripting-all"/>
        </xsl:element>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
           Outputs Plesk's sysuser element.
           Parameters:
           name     - login name
           password - encrypted password
           quota    - hard disk quota for this system user (in bytes)
           - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="sysuser">

        <xsl:param name="name"    />
        <xsl:param name="password"/>
        <xsl:param name="quota"   />

        <xsl:element name="sysuser">

          <xsl:attribute name="name">
            <xsl:value-of select="$name"/>
          </xsl:attribute>

          <xsl:if test="$quota">
            <xsl:attribute name="quota">
              <xsl:value-of select="$quota"/>
            </xsl:attribute>
          </xsl:if>

          <xsl:call-template name="password">
            <xsl:with-param name="password">
              <xsl:value-of select="$password"/>
            </xsl:with-param>
          </xsl:call-template>

        </xsl:element>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - -
           Outputs Plesk's password element.
           Parameters:
           password - encrypted or empty password
           password_type - password type (optional) by default assume type='encrypted'
           - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="password">
        <xsl:param name="password"/>
        <xsl:param name="password_type"/>
        <xsl:element name="password">
          <xsl:choose>
            <xsl:when test="string-length($password) > 0">
              <xsl:choose>
                <xsl:when test='$password_type="plain"'>
                  <xsl:attribute name="type">plain</xsl:attribute>
                </xsl:when>
                <xsl:otherwise>
                  <xsl:attribute name="type">encrypted</xsl:attribute>
                </xsl:otherwise>
              </xsl:choose>
              <xsl:value-of select="$password"/>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="type">empty</xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:element>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - - - - - - - -
           Outputs Plesk's scripting element with all scripting
           options enabled (used for webusers, hostings and
           subdomains).
           - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="scripting-all">
        <xsl:element name="scripting">
          <xsl:attribute name="ssi"   >true</xsl:attribute>
          <xsl:attribute name="php"   >true</xsl:attribute>
          <xsl:attribute name="cgi"   >true</xsl:attribute>
          <xsl:attribute name="perl"  >true</xsl:attribute>
          <!--		<xsl:attribute name="asp"   >true</xsl:attribute> -->
          <xsl:attribute name="python">true</xsl:attribute>
        </xsl:element>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - -
           Outputs Plesk's limit (used for domains
           and clients accounts)
           Parameters:
           limit - limit name
           value - limit value
           - - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="limit">
        <xsl:param name="limit"/>
        <xsl:param name="value"/>
        <xsl:element name="limit">
          <xsl:attribute name="name">
            <xsl:value-of select="$limit"/>
          </xsl:attribute>
          <xsl:choose>
            <xsl:when test="string($value)='' or string($value)='Infinity' or string($value)='NaN' or string($value)='-Infinity'">-1</xsl:when>
            <xsl:otherwise>
              <xsl:value-of select="format-number($value, '#')"/>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:element>
      </xsl:template>

      <!-- - - - - - - - - - - - - - - - - - - - - - -
           Outputs Plesk's permission
           Parameters:
           permission - permission name
           - - - - - - - - - - - - - - - - - - - - - - -->
      <xsl:template name="permission">
        <xsl:param name="permission"/>
        <xsl:element name="permission">
          <xsl:attribute name="name">
            <xsl:value-of select="$permission"/>
          </xsl:attribute>
        </xsl:element>
      </xsl:template>

      <!--
           cp4n312p135kcp4n312p135kcp4n312p135kcp4n312p135kcp4n312p135k

           Mail subsystem transformation

           cp4n312p135kcp4n312p135kcp4n312p135kcp4n312p135kcp4n312p135k
           -->
      <xsl:template name="mail" match="mail">

        <xsl:for-each select="mailname">

          <xsl:variable name="name"><xsl:value-of select="@name"/></xsl:variable>

          <mailuser name="{$name}" mailgroup-enabled="false">

            <!-- create password -->
            <xsl:if test="@password">
              <password type='encrypted'><xsl:value-of select="@password"/></password>
            </xsl:if>

            <!-- create mailbox: cPanel's mailnames always have mailboxes -->
            <xsl:element name="mailbox">
              <xsl:attribute name="type">mbox</xsl:attribute>
              <xsl:if test="@cid">
                <xsl:attribute name="cid">
                  <xsl:value-of select="@cid"/>
                </xsl:attribute>
              </xsl:if>
              <xsl:if test="@quota">
                <xsl:attribute name="quota">
                  <xsl:value-of select="@quota"/>
                </xsl:attribute>
              </xsl:if>
            </xsl:element>

            <!-- There may be forwarders and autoresponders in cPanel,
                 which may be mapped on the Plesk's business model:
                 mailname may have 1 forward and several autoresponders -->

            <xsl:if test="../forward[@mailname=$name]">
              <xsl:element name="redirect">
                <xsl:value-of select="../forward[@mailname=$name]/@redirect"/>
              </xsl:element>
            </xsl:if>

            <!-- convert only the From and subject parameters of the cPanel's autoresponder -->
            <xsl:if test="../autoresponder[@mailname=$name]">
              <xsl:element name="autoresponder">
                <xsl:attribute name="name">
                  <xsl:value-of select="$name"/>
                </xsl:attribute>
                <xsl:if test="../autoresponder[@mailname=$name]/@from">
                  <xsl:attribute name="replyto">
                    <xsl:value-of select="../autoresponder[@mailname=$name]/@from"/>
                  </xsl:attribute>
                </xsl:if>
                <xsl:if test="../autoresponder[@mailname=$name]/@subject">
                  <xsl:attribute name="subject">
                    <xsl:value-of select="../autoresponder[@mailname=$name]/@subject"/>
                  </xsl:attribute>
                </xsl:if>
                <xsl:value-of select="../autoresponder[@mailname=$name]"/>
              </xsl:element>
            </xsl:if>

          </mailuser>
        </xsl:for-each>

        <!-- There are separate autoresponders: convert'em to loginless mailnames -->
        <xsl:for-each select="autoresponder">

          <xsl:variable name="name"><xsl:value-of select="@mailname"/></xsl:variable>

          <!-- look that this autoresponder wasn't processed with mailname -->
          <xsl:if test="not(../mailname[@name=$name])">
            <mailuser name="{$name}" mailgroup-enabled="false">
              <autoresponder name="{$name}">
                <xsl:if test="string-length(@from) > 0">
                  <xsl:attribute name="replyto">
                    <xsl:value-of select="@from"/>
                  </xsl:attribute>
                </xsl:if>
                <xsl:if test="@subject">
                  <xsl:attribute name="subject">
                    <xsl:value-of select="@subject"/>
                  </xsl:attribute>
                </xsl:if>
                <xsl:value-of select="."/>
              </autoresponder>
            </mailuser>
          </xsl:if>

        </xsl:for-each>

        <!-- There are separate forwards: convert'em to loginless mailnames -->
        <xsl:for-each select="forward">
          <xsl:if test="not(../mailname[@name=current()/@name])">
            <mailuser name="{@mailname}" mailgroup-enabled="false">
              <redirect><xsl:value-of select="@redirect"/></redirect>
            </mailuser>
          </xsl:if>
        </xsl:for-each>

      </xsl:template>

    </xsl:stylesheet>
